﻿using ProjectSportCar.Entities;

namespace ProjectSportCar.Drawnings;

/// <summary>
/// Класс, отвечающий за прорисовку и перемещение объекта-сущности
/// </summary>
public class DrawningSportCar : DrawningCar
{
    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="bodyColor">Основной цвет</param>
    /// <param name="additionalColor">Дополнительный цвет</param>
    /// <param name="bodyKit">Признак наличия обвеса</param>
    /// <param name="wing">Признак наличия антикрыла</param>
    /// <param name="sportLine">Признак наличия гоночной полосы</param>
    public DrawningSportCar(int speed, double weight, Color bodyColor, Color additionalColor, bool bodyKit, bool wing, bool sportLine) : base(110, 60)
    {
        EntityCar = new EntitySportCar(speed, weight, bodyColor, additionalColor, bodyKit, wing, sportLine);
    }

	public override void DrawTransport(Graphics g)
	{
		if (EntityCar == null || EntityCar is not EntitySportCar sportCar || !_startPosX.HasValue || !_startPosY.HasValue)
		{
			return;
		}

		Pen pen = new(Color.Black);
		Brush additionalBrush = new SolidBrush(sportCar.AdditionalColor);

		// обвесы
		if (sportCar.BodyKit)
		{
			g.DrawEllipse(pen, _startPosX.Value + 90, _startPosY.Value, 20, 20);
			g.DrawEllipse(pen, _startPosX.Value + 90, _startPosY.Value + 40, 20, 20);
			g.DrawRectangle(pen, _startPosX.Value + 90, _startPosY.Value + 10, 20, 40);
			g.DrawRectangle(pen, _startPosX.Value + 90, _startPosY.Value, 15, 15);
			g.DrawRectangle(pen, _startPosX.Value + 90, _startPosY.Value + 45, 15, 15);

			g.FillEllipse(additionalBrush, _startPosX.Value + 90, _startPosY.Value, 20, 20);
			g.FillEllipse(additionalBrush, _startPosX.Value + 90, _startPosY.Value + 40, 20, 20);
			g.FillRectangle(additionalBrush, _startPosX.Value + 90, _startPosY.Value + 10, 20, 40);
			g.FillRectangle(additionalBrush, _startPosX.Value + 90, _startPosY.Value + 1, 15, 15);
			g.FillRectangle(additionalBrush, _startPosX.Value + 90, _startPosY.Value + 45, 15, 15);

			g.DrawEllipse(pen, _startPosX.Value, _startPosY.Value, 20, 20);
			g.DrawEllipse(pen, _startPosX.Value, _startPosY.Value + 40, 20, 20);
			g.DrawRectangle(pen, _startPosX.Value, _startPosY.Value + 10, 20, 40);
			g.DrawRectangle(pen, _startPosX.Value + 5, _startPosY.Value, 14, 15);
			g.DrawRectangle(pen, _startPosX.Value + 5, _startPosY.Value + 45, 14, 15);

			g.FillEllipse(additionalBrush, _startPosX.Value, _startPosY.Value, 20, 20);
			g.FillEllipse(additionalBrush, _startPosX.Value, _startPosY.Value + 40, 20, 20);
			g.FillRectangle(additionalBrush, _startPosX.Value + 1, _startPosY.Value + 10, 25, 40);
			g.FillRectangle(additionalBrush, _startPosX.Value + 5, _startPosY.Value + 1, 15, 15);
			g.FillRectangle(additionalBrush, _startPosX.Value + 5, _startPosY.Value + 45, 15, 15);

			g.DrawRectangle(pen, _startPosX.Value + 35, _startPosY.Value, 39, 15);
			g.DrawRectangle(pen, _startPosX.Value + 35, _startPosY.Value + 45, 39, 15);

			g.FillRectangle(additionalBrush, _startPosX.Value + 35, _startPosY.Value + 1, 40, 15);
			g.FillRectangle(additionalBrush, _startPosX.Value + 35, _startPosY.Value + 45, 40, 15);
		}

		_startPosX += 10;
		_startPosY += 5;
		base.DrawTransport(g);
		_startPosX -= 10;
		_startPosY -= 5;

		// спортивная линия
		if (sportCar.SportLine)
		{
			g.FillRectangle(additionalBrush, _startPosX.Value + 75, _startPosY.Value + 23, 25, 15);
			g.FillRectangle(additionalBrush, _startPosX.Value + 35, _startPosY.Value + 23, 35, 15);
			g.FillRectangle(additionalBrush, _startPosX.Value + 10, _startPosY.Value + 23, 20, 15);
		}

		// крыло
		if (sportCar.Wing)
		{
			g.FillRectangle(additionalBrush, _startPosX.Value, _startPosY.Value + 5, 10, 50);
			g.DrawRectangle(pen, _startPosX.Value, _startPosY.Value + 5, 10, 50);
		}
	}
}